@@ -306,22 +306,24 @@ def main(prog_or_class, singleton = SINGLETON_NOT_SINGLETON, multiproc_logging =
             exit_code = int(ret)
     except SystemExit, e:
         exit_code = e.code
-    except:
+    except BaseException, e:
         errinfo = traceback.format_exc()
         logger.critical(errinfo)
         
         exit_code, exit_message = prog.EXITCODE_MAIN_FAILURE
+        exit_message = str(e) or exit_message
         
     try:
         logger.info('Finalizing program.')
         prog.finalize()
     except SystemExit, e:
         exit_code = e.code
-    except:
+    except BaseException, e:
         errinfo = traceback.format_exc()
         logger.critical(errinfo)
         
         exit_code, exit_message = prog.EXITCODE_FINALIZATION_FAILURE
+        exit_message = str(e) or exit_message
         
     if exit_code:
         try:
